#!/usr/bin/python

""" CeibalSCAErrors -- Ceibal SCA library. Exception classes.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id: CeibalSCAErrors.py 122 2009-05-16 19:39:07Z jcampo $
"""

class CeibalSCAException(Exception):
    """ CeibalSCA generic Exception.
    """
    internalMSG = ""
    
    def __init__(self, message=""):
        self.internalMSG = message

    def __str__(self):
        return "CeibalSCA Generic Exception: %s." % self.internalMSG

class CeibalSCAXMLException(CeibalSCAException):
    def __str__(self):
        return "Error parsing embedded XML: %s" % self.internalMSG

class CeibalSCAProtocolException(CeibalSCAException):
    def __str__(self):
        return "Protocol error: %s" % self.internalMSG

class CeibalSCAHTTPException (CeibalSCAException):
    """ Represents an error in the HTTP communication with the Web
        Server. Could be caused by networking problems too.
    """
    def __init__(self, message="", http_code=None):
        self.internalMSG = message
        self.http_code = http_code
        
    def __str__(self):
        string = "HTTP Error: %s." % self.internalMSG
        if (not self.http_code is None):
            string = string + "\n  " + "Server returned %d code." % self.http_code
        return string

class CeibalSCASeedExists (CeibalSCAException):
    """ Seed found during registration phase.
    """
    def __str__(self):
        return "Seed exists, cannot register"
    

class WisprException (CeibalSCAException):
    """ WISPr generic Exception.
    """
    def __str__(self):
        return "WISPr Generic Exception: %s." % self.internalMSG

class WisprProtocolException (WisprException):
    """ Represents an error in the WISPr protocol. Could be caused by
        a missing embedded XML or server replies error codes.
    """
    def __init__(self, message="", response_code=None):
        self.internalMSG = message
        self.response_code = response_code

    def __str__(self):
        string = "WISPr Protocol Error: %s." % self.internalMSG
        if (not self.response_code is None):
            string = string + "\n  " + "Server response code is: %d" % self.response_code
        return string
        
class WisprLoginFailedException(WisprProtocolException):
    """ A failed login attempt, probably because credentials were
        invalid.

        Server side errors (like RADIUS errors) are handled by
        WisprProtocolException.
    """
    def __str__(self):
        return "WISPr Login Failed: %s." % self.internalMSG

class WisprAlreadyLoggedinException(WisprProtocolException):
    """ Tried to authenticate when we were already logged in.
    """
    def __str__(self):
        return "WISPr Login Failed: Already logged in."

        
