#!/usr/bin/env python

from M2Crypto import Rand, SSL, httpslib, m2urllib2
import urllib2, httplib

from CeibalSCAErrors import *
from CeibalSCAConf import *
from CeibalSCALog import *
import re

CEIBAL_HTTP_VERSION = 1.0
HTTPS_PORT_DEFAULT = 443
RANDPOOL = 'randpool.dat'

class CeibalHTTPRedirectHandler(m2urllib2.HTTPRedirectHandler):
	def __init__(self): pass

	def http_error_302(self,req, fp, code, msg, hdrs):
		return fp

class CeibalHTTP():
	def __init__(self):
		self.log = CeibalSCALog.get_instance()
		self.followRedirect = True 
		#self.userAgent = [('User-agent', 'CeibalHTTP/' + str(CEIBAL_HTTP_VERSION))]
		self.userAgent = 'CeibalHTTP/' + str(CEIBAL_HTTP_VERSION)

		self.ceibal_conf = CeibalSCAConf.get_instance()
		self.ceibal_conf.add_config_changed_callback(self._configure_ssl_context)
		
		self._configure_ssl_context()

		
	def _configure_ssl_context(self):
		self.ctx = None
		if (self.ceibal_conf.get_check_ca()):
			self.ctx = SSL.Context('sslv23')
			depth = self.ceibal_conf.get_ca_certificate_depth()
			self.ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert , depth)
			if self.ctx.load_verify_locations(self.ceibal_conf.get_ca_certificate()) != 1:
				raise CeibalSCAException('CA certificates not found')

	def setFollowRedirect(self, boolean = True): self.followRedirect = boolean
	def getFollowRedirect(self): return self.followRedirect

	def open(self, url):
		""" Open the URL url, which can be either a string or
		    a Request object. url (string): Can be a request
		    object or a string. Return: An addinfourl
		    instance.
		"""
		if isinstance(url, basestring):
			req = urllib2.Request(url)
		else:
			req = url

		if (req.has_data()):
			method='POST'
		else:
			method='GET'

		self.log.debug(req.get_full_url())
		if (re.search("^https://", req.get_full_url())):
			if (self.ctx):
				https_connection = httpslib.HTTPSConnection (host = req.get_host(), ssl_context = self.ctx)
			else:
				https_connection = httpslib.HTTPSConnection (host = req.get_host())
		else:
			https_connection = httplib.HTTPConnection (host = req.get_host())

		string = "Cannot open URL %s" % req.get_full_url()
		try:
			headers = {}
			headers['Content-Type'] = "application/x-www-form-urlencoded"
			headers['User-Agent'] = self.userAgent
			https_connection.request (method, req.get_selector(), req.get_data(), headers)

			http_response = https_connection.getresponse()

			status = http_response.status
			if (status in (301, 302, 307)):
				if (self.followRedirect):
					new_url = http_response.getheader('location')
					return self.open (urllib2.Request (new_url, req.get_data()))
				else:
					return http_response
			elif (status < 200 or status > 209):
				raise CeibalSCAHTTPException (string, status)
			else:
				return http_response
		except CeibalSCAHTTPException, e:
			raise e
		except Exception, e:
			string = str(e)
			raise CeibalSCAHTTPException(string)


	def post_data (self, url, data=None):
		""" Sends an http request to the given URL. If data is
		    not None, make a POST. Returns a handler to the
		    resulting page.

		    Raises CeibalSCAException on error.

		    Arguments:
		      + The url to POST to.
		      + The optional data to POST. Data must be a
		        dictionary, with name, value of the data to
		        POST.
		"""
		if (data):
			post_str = ""
			for k, v in data.iteritems():
				if (post_str != ""):
					post_str = post_str + "&"
				post_str = post_str + "%s=%s" % (k, v)
			return self.open(urllib2.Request(url, post_str))
		else:
			return self.open(url)
	
