#!/usr/bin/python

""" CeibalSCASeed -- Ceibal SCA XO client, Seed classes.

    Written by Mario Del Riego <mriego@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id: CeibalSCASeed.py 431 2009-06-20 16:02:06Z jcampo $
"""

# States
STATE_INITIAL       = 0
STATE_SYNCHRONIZED  = 1
STATE_RESET         = 2

import pickle, os
from CeibalSCAConf import *

class Seed:
    # List of known seeds, which are allowed to persist in
    # filesystem.
    _names = ['FirstSeed','CurrentSeed']

    def __init__(self, name=None, state = STATE_INITIAL, value = None,
		 count = 0, refresh_interval = 0, last_used = None,
		 times_used = 0, errors = 0):
	# Seed Name, if it has one
	self._name = name
	# State (int)
	self._state = state
	# Seed value (string)
	self._value = value
	# Integer's
	self._count = count
	self._refresh_interval = refresh_interval
        self._max_login_fail = 0
        self._max_resync_fail = 0
        self._max_reauth_fail = 0
        self._max_sync_fail = 0
        self._session_timeout = 0

        self._reauth_timestamp = 0

	self._last_used = last_used
	self._times_used = times_used
        self._errors = errors

    # Set/Get of value
    def getValue(self):
	return self._value
    def setValue(self,value):
	self._value = value

    # State functions
    def setStateInitial(self): self.setState(STATE_INITIAL)
    def setStateSynchronized(self): self.setState(STATE_SYNCHRONIZED)
    def setStateReset(self): self.setState(STATE_RESET)

    def getState(self): return (self._state)
    def setState(self, st): self._state = st
	 
    # Set/Get of refresh_interval 
    def getInterval(self):
	return self._refresh_interval
    def setInterval(self, interval):
	self._refresh_interval = interval

    # Set/Get of max_XXX_fail
    def setMaxLoginFail(self, max_login_fail):
        self._max_login_fail = max_login_fail
    def setMaxResyncFail(self, max_resync_fail):
        self._max_resync_fail = max_resync_fail
    def setMaxReauthFail(self, max_reauth_fail):
        self._max_reauth_fail = max_reauth_fail
    def setMaxSyncFail(self, max_sync_fail):
        self._max_sync_fail = max_sync_fail

    def getMaxLoginFail(self):
        return self._max_login_fail
    def getMaxResyncFail(self):
        return self._max_resync_fail
    def getMaxReauthFail(self):
        return self._max_reauth_fail
    def getMaxSyncFail(self):
        return self._max_sync_fail

    # Set/Get of session timeout
    def setSessionTimeout(self, timeout):
        self._session_timeout = timeout
    def getSessionTimeout(self):
        return self._session_timeout

    # Increase count by one
    def incrCount(self): self._count = self._count + 1
    def resetCount(self): self._count = 0
    def getCount(self): return self._count

    # Reauth Timestamp functions
    def incrReauthTimestamp(self):
        self._reauth_timestamp += 1
    def resetReauthTimestamp(self):
        self._reauth_timestamp = 0
    def getReauthTimestamp(self):
        return self._reauth_timestamp

    # Set/Get of last_used
    def setLastUsed(self, current_time):
	self._last_used = current_time
    def getLastUsed(self):
	return self._last_used

    # Set/Get of times_used
    def setTimesUsed(self, times_used):
	self._times_used = times_used
    def incrTimesUsed(self):
	self._times_used = self._times_used + 1
    def getTimesUsed(self):
	return self._times_used

    # Set/Get of errors
    def setErrors(self, errors): self._errors = errors
    def incrErrors(self): self._errors = self._errors + 1
    def getErrors(self): return self._errors

    def save(self):
	if (self._name in self._names):
            ceibal_conf = CeibalSCAConf.get_instance()
            file_name = os.path.join(ceibal_conf.get_sca_security_path(),
                                     self._name)
            
	    pkl_file = open(file_name, 'wb')
	    pickle.dump(self, pkl_file)
	    pkl_file.close()
	else:
	    return None

    def load(self): 
	if (not self._name in self._names):
            return None
        try:
            ceibal_conf = CeibalSCAConf.get_instance()
            file_name = os.path.join(ceibal_conf.get_sca_security_path(),
                                     self._name)

	    pkl_file = open(file_name, 'rb')
	    tmp = pickle.load(pkl_file)
	    pkl_file.close()
	    return tmp
	except:
	    return None

    def delete(self):
        """ Deletes the seed from the filesystem. Use with care.

            Arguments: None
        """
        if (not self._name in self._names):
            return None
        try:
            ceibal_conf = CeibalSCAConf.get_instance()
            file_name = os.path.join(ceibal_conf.get_sca_security_path(),
                                     self._name)
            os.remove(file_name)
        except:
            return None

class FirstSeed(Seed):
    def __init__(self):
	Seed.__init__(self, 'FirstSeed')

class CurrentSeed(Seed):
    def __init__(self):
	Seed.__init__(self, 'CurrentSeed')


