#!/usr/bin/python

import gobject, time

class CeibalSCATimer:
    """ Ceibal SCA XO client timer. This class is a
        SINGLETON, please don't call constructor directly. Use
        CeibalSCATimer.get_instance(), instead.
    """

    instance = None
    
    class Singleton:
        """ Auxiliary class """
        def __call__( self, *args, **kw ) :

            # If an instance of CeibalSCATimer does not exist, create
            # one and assign it to CeibalSCATimer.instance.

            if (CeibalSCATimer.instance is None):
                object = CeibalSCATimer()
                CeibalSCATimer.instance = object

            # Return CeibalSCATimer.instance, which should contain a
            # reference to the only instance of CeibalSCATimer in the
            # system.

            return CeibalSCATimer.instance

    get_instance = Singleton()


    def __init__ (self):
        """ DO NOT CALL THIS METHOD! """
        self.timer_id = None

    def set_timer(self, timeout, callback):
        """ Set a timer. Call callback in timeout seconds.
            Removes any existing timer.
        """
        self.cancel_timer()
        self.timeout = timeout
        self.callback = callback
        self.timer_id = gobject.timeout_add(timeout*1000, callback)

    def cancel_timer(self):
        """ Cancels the timer.
        """
        if (self.timer_id):
            gobject.source_remove(self.timer_id)
            self.timer_id = None
            self.timeout = None
            self.callback = None

    def reset_timer(self):
        """ Resets a previously set timer. It has the same interval
            and callback as the previous one. If no previous timer was
            set, do nothing.
        """
        if (self.timer_id):
            gobject.source_remove(self.timer_id)
            
        if ((not self.timer_id) or (not self.timeout) or
            (not self.callback)):
            return

        self.timer_id = gobject.timeout_add(self.timeout*1000, self.callback)
        

