#!/usr/bin/python

""" CeibalSCAXML -- Ceibal SCA XO client, embedded XML parsing

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id$
"""

from xml.dom.ext.reader import Sax2
import xml.dom
import re

from CeibalSCAErrors import *

class CeibalSCAXML:
    def read_embedded_xml(self, html_contents, top_tag):
        """ Extracts the embedded XML from a HTML page, and returns
            the an object representing the given tag.

            Raises a CeibalSCAXMLException if no XML is found, or if
            there is an error parsing the XML.

            Arguments:
              + The contents of the HTML page as a string.
              + The top tag to search for (must be unique).
        """

        m = re.search('<!--[^<]*(<\?xml[^?]*\?>.*)-->', html_contents, re.DOTALL)
        if (not m):
            raise CeibalSCAXMLException("No embedded XML found")
        
        xml_contents = m.group(1)
        
        try:
            xml_reader = Sax2.Reader()
            xml_dom = xml_reader.fromString(xml_contents)
            tag_list = xml_dom.getElementsByTagName(top_tag)
            if (tag_list.length == 0):
                raise CeibalSCAXMLException("%s tag not found" % top_tag)
            elif (tag_list.length > 1):
                raise CeibalSCAXMLException("More than one %s tag" % top_tag)

            ret_tag = tag_list[0]
            
        except xml.sax.SAXException, e:
            raise CeibalSCAXMLException(e.message)
        except xml.dom.DOMException, e:
            raise CeibalSCAXMLException(e.message)

        return ret_tag

    def get_unique_text_value(self, top_tag, tag_name, opt=False):

        """ Returns the text value of a given XML tag.

            If there are more than one tag with that name, or it is
            not a text tag, raise an CeibalSCAXMLException. If opt
            argument is False (default), raise an exception if tag is
            not found.

            Arguments:
              + The top XML tag object.
              + The tag name to search for
              + An optional arg. If this is True, don't raise Exception if
                tag is not found. Default is False.
        """

        tag_list = top_tag.getElementsByTagName(tag_name)
        if (not tag_list or tag_list.length == 0):
            if (not opt):
                raise CeibalSCAXMLException ("Tag %s not found" % tag_name)
            else:
                return None
        if (tag_list.length > 1):
            raise CeibalSCAXMLException ("More than one %s tag" % tag_name)
        tag = tag_list[0]
        if (tag.nodeType != xml.dom.Node.ELEMENT_NODE):
            raise CeibalSCAXMLException ("%s is not an ELEMENT_NODE" % tag_name)
        if (tag.childNodes.length != 1):
            raise CeibalSCAXMLException ("%s is not a TEXT tag" % tag_name)
        if (tag.firstChild.nodeType != xml.dom.Node.TEXT_NODE):
            raise CeibalSCAXMLException ("%s is not a TEXT tag" % tag_name)

        return str (tag.firstChild.nodeValue)

