#!/usr/bin/python

""" CeibalSCA_NM_0_6_5 -- Handles NetworkManager communication for the
    Ceibal SCA XO client. Only for version 0.6.5 of NM.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id$

    
"""

import dbus, re, sys, platform, time
from dbus.mainloop.glib import DBusGMainLoop

from CeibalSCALog import *
from CeibalSCAErrors import *
from nm_constants import *
from CeibalSCAConf import *
from CeibalSCA_NM import *

class CeibalSCA_NM_0_6_5(CeibalSCA_NM):

    def __init__(self, connect_callback, disconnect_callback):
        """ Create an instance of CeibalSCA_NM class.

            Arguments:
              + Callback function to call when connected to a Ceibal
                network. Must receive no arguments.
              + Callback function to call when disconnected of a Ceibal
                network. Must receive no arguments.
        """
        CeibalSCA_NM.__init__(self, connect_callback, disconnect_callback)
        DBusGMainLoop(set_as_default=True)
        self.system_bus = dbus.SystemBus()
        self.network_manager = dbus.Interface (
            self.system_bus.get_object(NM_DBUS_SERVICE, NM_DBUS_PATH),
            dbus_interface=NM_DBUS_INTERFACE)
        self.ceibalsca_dbus = dbus.Interface (
            self.system_bus.get_object(CEIBALSCA_DBUS_SERVICE, CEIBALSCA_DBUS_PATH),
            dbus_interface=CEIBALSCA_DBUS_INTERFACE)
        self.ceibalsca_dbus.connect_to_signal(NM_DBUS_SIGNAL_STATE_CHANGE, self._nm_state_changed)
        

    def _get_nm_connection_params(self):
        """ This function returns a dictionary with the following data:
              "if_name":      The interface name used in the NetworkManager connection
              "network_name": The SSID of the connected network.
              "ip_addr":      The assigned IP address, as returned by NM (integer).

            If the interface is not connected, return None.
        """
        device_interface = dbus.Interface (
            self.system_bus.get_object(NM_DBUS_SERVICE,
                                       NM_DBUS_PATH_DEVICES+"/"+self.ceibal_conf.get_network_interface()),
            dbus_interface=NM_DBUS_INTERFACE_DEVICES)

        if (device_interface.getType() != DEVICE_TYPE_802_11_WIRELESS):
            # Ignore it, if it isn't a wireless device
            return None

        try:
            network_interface = device_interface.getActiveNetwork()
        except dbus.DBusException, e:
            if (nm_dbus_exception(e, "NoActiveNetwork") or nm_dbus_exception(e, "UnknownMethod")):
                # If our interface is not active, ignore it.
                return None
            else:
                raise e

        m = re.search('^.*/(.*)$', network_interface)
        if (not m):
            raise CeibalSCAException("Could not parse NM active network")
        
        params = {'if_name': device_interface.getName(),
                  'network_name': m.group(1),
                  'ip_addr': device_interface.getIP4Address()}
        return params
        


    # PRIVATE FUNCTIONS
    # =================

    def _nm_state_changed (self, state):
        if (state == NM_STATE_DISCONNECTED):
            self._nm_disconnected()
        if (state == NM_STATE_CONNECTED):
            self._nm_connected()
