#!/usr/bin/python

""" CeibalSCA_NM_Factory -- Creates CeibalSCA_NM objects, for the
    appropriate NetworkManager version.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    June 2009.

    VERSION: $Id$

    To create a new handler for another version of NM, implement a
    derived class of CeibalSCA_NM, and add it here, in get_nm_handler.
"""

import os, re
from CeibalSCA_NM import *
from CeibalSCA_NM_0_6_5 import *
from CeibalSCA_NM_0_7_2 import *
from CeibalSCA_NM_0_8_4 import *
from CeibalSCA_NM_0_8_5 import *

from CeibalSCALog import *

def get_nm_handler(connect_callback, disconnect_callback):
    """ Create an instance of CeibalSCA_NM class, for the appropriate
        NM version.

        Arguments:
          + Callback function to call when connected to a Ceibal
            network. Must receive no arguments.
          + Callback function to call when disconnected of a Ceibal
            network. Must receive no arguments.
    """
    version = _get_nm_version()

    log = CeibalSCALog.get_instance()
    log.debug("Detected version of Network Manager: %s" %  str(version))

    if (re.search("^0\.6\.5", version)):
        return CeibalSCA_NM_0_6_5(connect_callback, disconnect_callback)
    elif (re.search("^0\.7\.2", version)):
         return CeibalSCA_NM_0_7_2(connect_callback, disconnect_callback)
    elif (re.search("^0\.8\.4", version)):
         return CeibalSCA_NM_0_8_4(connect_callback, disconnect_callback)
    elif (re.search("^0\.8\.5", version)):
         return CeibalSCA_NM_0_8_5(connect_callback, disconnect_callback)
    else:
        raise CeibalSCAException("NetworkManager version %s not supported" % version)


def _get_nm_version():
    version = os.popen("rpm -q NetworkManager").readline()
    m = re.search('^NetworkManager-(\d+\.\d+\.\d+).*\..*\..*$', version)
    if (not m):
        raise CeibalSCAException("Could not parse NM version")

    return m.group(1)
