#  NetworkManager -- Network link manager
#
#  Dan Williams <dcbw@redhat.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#  (C) Copyright 2004 Red Hat, Inc.

# CeibalSCA DBUS API
CEIBALSCA_DBUS_PATH         = "/org/freedesktop/CeibalSCASignal"
CEIBALSCA_DBUS_INTERFACE    = "org.freedesktop.CeibalSCASignal.HelloWorldIFace"
CEIBALSCA_DBUS_SERVICE      = 'org.freedesktop.CeibalSCA'

# dbus services details
NM_DBUS_SERVICE                 = "org.freedesktop.NetworkManager"

NM_DBUS_PATH                    = "/org/freedesktop/NetworkManager"
NM_DBUS_INTERFACE               = "org.freedesktop.NetworkManager"
NM_DBUS_PATH_DEVICES            = "/org/freedesktop/NetworkManager/Devices"
NM_DBUS_INTERFACE_DEVICES       = "org.freedesktop.NetworkManager.Devices"
NM_DBUS_INTERFACE_AP            = "org.freedesktop.NetworkManager.AccessPoint"
NM_DBUS_PATH_DHCP               = "/org/freedesktop/NetworkManager/DhcpOptions"
NM_DBUS_INTERFACE_DHCP          = "org.freedesktop.NetworkManager.DhcpOptions"
NM_DBUS_PROPERTIES              = "org.freedesktop.DBus.Properties"

NMI_DBUS_SERVICE                = "org.freedesktop.NetworkManagerInfo"
NMI_DBUS_PATH                   = "/org/freedesktop/NetworkManagerInfo"
NMI_DBUS_INTERFACE              = "org.freedesktop.NetworkManagerInfo"


# Some common errors
NM_DBUS_NO_DEVICES_ERROR        = "org.freedesktop.NetworkManager.NoDevices"
NM_DBUS_NO_DIALUP_ERROR         = "org.freedesktop.NetworkManager.NoDialup"
NM_DBUS_NO_NETWORKS_ERROR       = "org.freedesktop.NetworkManager.NoNetworks"
NM_DBUS_NO_ACTIVE_DEVICE_ERROR  = "org.freedesktop.NetworkManager.NoActiveDevice"
NM_DBUS_NO_ACTIVE_NET_ERROR     = "org.freedesktop.NetworkManager.NoActiveNetwork"

NMI_DBUS_USER_KEY_CANCELED_ERROR = "org.freedesktop.NetworkManagerInfo.CanceledError"


# NetworkManager signals
NM_DBUS_SIGNAL_STATE_CHANGE = "StateChange"


# Types of NetworkManager states
NM_STATE_UNKNOWN        = 0
NM_STATE_ASLEEP         = 1
NM_STATE_CONNECTING     = 2
NM_STATE_CONNECTED      = 3
NM_STATE_DISCONNECTED   = 4


# Types of NetworkManager devices
DEVICE_TYPE_UNKNOWN             = 0
DEVICE_TYPE_802_3_ETHERNET      = 1
DEVICE_TYPE_802_11_WIRELESS     = 2


# General device capability bits
NM_DEVICE_CAP_NONE              = 0x00000000
NM_DEVICE_CAP_NM_SUPPORTED      = 0x00000001
NM_DEVICE_CAP_CARRIER_DETECT    = 0x00000002
NM_DEVICE_CAP_WIRELESS_SCAN     = 0x00000004


# 802.11 wireless-specific device capability bits
NM_802_11_CAP_NONE              = 0x00000000
NM_802_11_CAP_PROTO_NONE        = 0x00000001
NM_802_11_CAP_PROTO_WEP         = 0x00000002
NM_802_11_CAP_PROTO_WPA         = 0x00000004
NM_802_11_CAP_PROTO_WPA2        = 0x00000008
NM_802_11_CAP_RESERVED1         = 0x00000010
NM_802_11_CAP_RESERVED2         = 0x00000020
NM_802_11_CAP_KEY_MGMT_PSK      = 0x00000040
NM_802_11_CAP_KEY_MGMT_802_1X   = 0x00000080
NM_802_11_CAP_RESERVED3         = 0x00000100
NM_802_11_CAP_RESERVED4         = 0x00000200
NM_802_11_CAP_RESERVED5         = 0x00000400
NM_802_11_CAP_RESERVED6         = 0x00000800
NM_802_11_CAP_CIPHER_WEP40      = 0x00001000
NM_802_11_CAP_CIPHER_WEP104     = 0x00002000
NM_802_11_CAP_CIPHER_TKIP       = 0x00004000
NM_802_11_CAP_CIPHER_CCMP       = 0x00008000

# NM-supported Authentication Methods
NM_AUTH_TYPE_WPA_PSK_AUTO       = 0x00000000
NM_AUTH_TYPE_NONE               = 0x00000001
NM_AUTH_TYPE_WEP40              = 0x00000002
NM_AUTH_TYPE_WPA_PSK_TKIP       = 0x00000004
NM_AUTH_TYPE_WPA_PSK_CCMP       = 0x00000008
NM_AUTH_TYPE_WEP104             = 0x00000010
NM_AUTH_TYPE_WPA_EAP            = 0x00000020
NM_AUTH_TYPE_LEAP               = 0x00000040


# EAP Method in libnm-util is a bitfield of (EAP Method) | (Phase2 Method)
NM_EAP_METHOD_MASK              = 0x0000ffff
NM_PHASE2_METHOD_MASK           = 0xffff0000

def NM_EAP_TO_EAP_METHOD(eap):
    return (eap & NM_EAP_METHOD_MASK)

def NM_EAP_TO_PHASE2_METHOD(eap):
    return (eap & NM_PHASE2_METHOD_MASK)

# EAP Methods
NM_EAP_METHOD_MD5               = 0x00000001    # EAP-MD5
NM_EAP_METHOD_MSCHAP            = 0x00000002    # EAP-MSCHAPv2
NM_EAP_METHOD_OTP               = 0x00000004    # EAP-OTP
NM_EAP_METHOD_GTC               = 0x00000008    # EAP-GTC
NM_EAP_METHOD_PEAP              = 0x00000010    # EAP-PEAP
NM_EAP_METHOD_TLS               = 0x00000020    # EAP-TLS
NM_EAP_METHOD_TTLS              = 0x00000040    # EAP-TTLS

# Phase2 Methods
NM_PHASE2_AUTH_NONE             = 0x00000000
NM_PHASE2_AUTH_PAP              = 0x00010000
NM_PHASE2_AUTH_MSCHAP           = 0x00020000
NM_PHASE2_AUTH_MSCHAPV2         = 0x00030000
NM_PHASE2_AUTH_GTC              = 0x00040000


# Wireless network update types
NETWORK_STATUS_DISAPPEARED      = 0
NETWORK_STATUS_APPEARED         = 1
NETWORK_STATUS_STRENGTH_CHANGED = 2


# Wireless network types
NETWORK_TYPE_UNKNOWN = 0
NETWORK_TYPE_ALLOWED = 1
NETWORK_TYPE_INVALID = 2
NETWORK_TYPE_DEVICE  = 3


# Device activation stages
NM_ACT_STAGE_UNKNOWN            = 0
NM_ACT_STAGE_DEVICE_PREPARE     = 1
NM_ACT_STAGE_DEVICE_CONFIG      = 2
NM_ACT_STAGE_NEED_USER_KEY      = 3
NM_ACT_STAGE_IP_CONFIG_START    = 4
NM_ACT_STAGE_IP_CONFIG_GET      = 5
NM_ACT_STAGE_IP_CONFIG_COMMIT   = 6
NM_ACT_STAGE_ACTIVATED          = 7
NM_ACT_STAGE_FAILED             = 8
NM_ACT_STAGE_CANCELLED          = 9

def nm_dbus_exception (e, guessed_exception):
    """Checks if the DBus exception e is (exactly) of type guessed_exception"""
    try:
        return e.get_dbus_name().endswith(guessed_exception)
    except:
        # If it doesn't have a get_dbus_name, it probably isn't the DBus
        # exception we're looking for.
        return False
