#!/usr/bin/python

""" CeibalSCA -- Ceibal SCA XO client main program.

    Written by Juan Diego Campo <jcampo@tilsor.com.uy>, for TILSOR S.A.
    Ceibal, LATU SCA Project
    April 2009.

    VERSION: $Id: CeibalSCA.py 673 2009-07-30 17:02:49Z jcampo $
"""

import sys, gobject, signal, time

from CeibalSCAProtocol import *
from CeibalSCAErrors import *
from CeibalSCALog import *
from CeibalSCA_NM_Factory import *
from CeibalSCAState import *
from CeibalSCASeed import *
from CeibalSCAUtils import *
from CeibalSCATimer import *

def connection_handler():
    """ Handler called when NetoworkManager is connected to the Ceibal
        network.
    """
    log = CeibalSCALog.get_instance()

    # If there was a reauth timer, delete it.
    reauth_timer.cancel_timer()

    ceibal_conf = CeibalSCAConf.get_instance()

    first_seed = FirstSeed()
    current_seed = CurrentSeed()
    first_seed = first_seed.load()
    current_seed = current_seed.load()

    if ((not first_seed) or (not current_seed)):
        # No first or current seed. We have to register.
        client_state.set_state ("Seeds not found", CS_INACT_NOTREG)
        first_seed = FirstSeed()
        current_seed = CurrentSeed()

    try:
        if (client_state.equal(CS_REAUTHNG_SYNC)):
            # We were trying to reauthenticate, and now WLAN is Up. Go
            # to AUTH_SYNC to authenticate (if needed) or
            # reauthenticate
            client_state.set_state("Reauthenticating and WLAN Up", CS_AUTH_SYNC)

        if (client_state.equal(CS_AUTH_SYNC)):
            # WLAN Up and we are authenticated, check we can reach
            # public url, and reauthenticate to reset timeout.
            if (proto.check_public_url()):
                timer_handler()
                return
            else:
                # Cannot reach public_url, authenticate once more.
                client_state.set_state("Not auth", CS_ACT_SYNC)

        # Retry until we are authenticated and syncrhonized.
        while (not client_state.equal(CS_AUTH_SYNC)):

            if (client_state.equal(CS_INACT_NOTREG)):
                client_state.set_state("Wlan UP", CS_ACT_NOTREG)

            if (client_state.equal(CS_INACT_DESYNC)):
                client_state.set_state("Wlan UP", CS_ACT_DESYNC)

            if (client_state.equal(CS_INACT_SYNC)):
                client_state.set_state("Wlan UP", CS_ACT_SYNC)

            if (client_state.equal(CS_ACT_NOTREG)):
                # Kind of useless state. Keep it to document.
                client_state.set_state("Register", CS_ACT_REGNG)

            while (client_state.equal(CS_ACT_REGNG)):
                # TODO - This could be an ugly infinite loop, if we
                # are not able to register for any reason. Maybe we
                # should break after enough retries.
                try:
                    proto.register()
                    client_state.set_state("Registration OK", CS_ACT_SYNC)
                except CeibalSCASeedExists, e:
                    # Seed exists, not registering.
                    log.error(str(e))
                    client_state.set_state("Seed exists", CS_ACT_SYNC)
                except Exception, e:
                    log.error(str(e))
                    # Wait for 10 seconds to retry registration
                    time.sleep(ceibal_conf.get_register_sleep())
                    if (not nm_handler.check_correct_network()):
			# If there was a reauth timer, delete it.
			reauth_timer.cancel_timer()
                        # We are not connected. Exit and don't get called again.
                        client_state.set_state("Wlan Down", CS_INACT_NOTREG)
			return

            if (client_state.equal(CS_ACT_SYNC)):
                client_state.set_state("Authenticate", CS_AUTHNG_SYNC)

            while (client_state.equal(CS_AUTHNG_SYNC)):
                current_seed = current_seed.load()

                try:
                    last_used = current_seed.getLastUsed()
                    proto.authenticate()
                    current_seed = current_seed.load()
                    current_seed.setErrors(0)
                    current_seed.save()

                    # We are authenticated. Check if we have to synchronize
                    if ((current_seed.getCount() >= current_seed.getTimesUsed()) or
                        (last_used + current_seed.getInterval()*3600 < getCurrentTime())):
                        client_state.set_state("Auth OK, sync", CS_AUTH_SYNCNG)
                    else:
                        client_state.set_state("Auth OK, no sync", CS_AUTH_SYNC)

                    # Set reauth timeout. Seed timeout is in
                    # minutes. set_timer expects seconds.
                    # We divide by 2.
                    reauth_timer.set_timer(current_seed.getSessionTimeout()*30, timer_handler)

                except WisprAlreadyLoggedinException, e:
                    # Reauth to reset session timeout
                    client_state.set_state("Already logged in", CS_AUTH_SYNC)
                    current_seed = current_seed.load()
                    current_seed.setErrors(0)
                    current_seed.save()
                    timer_handler()
                    # We have to return, as timer_handler already
                    # called this function.
                    return
                except Exception, e:
                    log.error(str(e))
                    current_seed.incrErrors()
                    if (current_seed.getErrors() >= current_seed.getMaxLoginFail()):
                        client_state.set_state("Too many auth errors", CS_ACT_DESYNC)
                        # Reset errors, as we are going to resynchronize.
                        current_seed.setErrors(0)
                        current_seed.save()
                        break
                    else:
                        current_seed.save()
                        time.sleep(ceibal_conf.get_auth_sleep()) # Wait before retry

            while (client_state.equal(CS_AUTH_SYNCNG)):
                current_seed = current_seed.load()
                try:
                    proto.synchronize()
                    current_seed = current_seed.load()
                    current_seed.setErrors(0)
                    current_seed.save()

                    client_state.set_state("Sync OK", CS_AUTH_SYNC)
                except Exception, e:
                    log.error(str(e))
                    current_seed.incrErrors()
                    if (current_seed.getErrors() >= current_seed.getMaxSyncFail()):
                        client_state.set_state("Too many sync errors", CS_ACT_DESYNC)
                        current_seed.setErrors(0)
                        current_seed.save()
                        break
                    else:
                        current_seed.save()
                        time.sleep(ceibal_conf.get_sync_sleep()) # Wait before retry

            if (client_state.equal(CS_ACT_DESYNC)):
                # Resynchronization needed
                client_state.set_state("Resyncronize", CS_ACT_RESYNCNG)

            while (client_state.equal(CS_ACT_RESYNCNG)):
                current_seed = current_seed.load()
                try:
                    proto.resynchronize()
                    current_seed = current_seed.load()
                    client_state.set_state("Resyncronization OK", CS_ACT_SYNC)
                    current_seed.setErrors(0)
                    current_seed.save()
                except Exception, e:
                    log.error(e)
                    current_seed.incrErrors()
                    if (current_seed.getErrors() >= current_seed.getMaxResyncFail()):
                        # Game Over. Start again from the begining
                        # (register once more).
                        client_state.set_state("Too many resync errors", CS_ACT_NOTREG)
                        first_seed.delete()
                        current_seed.delete()
                        break
                    else:
                        current_seed.save()
                        time.sleep(ceibal_conf.get_resync_sleep()) # Wait before retry

        # end of while client_state != CS_AUTH_SYNC
    except Exception, e:
        log.error(str(e))

def disconnection_handler():
    """ Handler called when NetoworkManager is diconnected from the
        Ceibal network.
    """
    # If there was a reauth timer, delete it.
    reauth_timer.cancel_timer()

    if (client_state.equal(CS_INACT_NOTREG)):
	return

    if (client_state.equal(CS_ACT_NOTREG) or
        client_state.equal(CS_ACT_REGNG)):
        client_state.set_state("Wlan Down", CS_INACT_NOTREG)
    elif (client_state.equal(CS_ACT_RESYNCNG) or
          client_state.equal(CS_ACT_DESYNC)):
        client_state.set_state("Wlan Down", CS_INACT_DESYNC)
    else:
        client_state.set_state("Wlan Down", CS_INACT_SYNC)

def timer_handler():
    """ Handler called when a timeout of half the session time occurs.
        Return True if we have to reset timer, False otherwise.
    """
    # Always cancel timer, and set it again if necesary, to avoid
    # getting called too fast.
    reauth_timer.cancel_timer()

    log = CeibalSCALog.get_instance()
    log.debug("nm_handler.check_correct_network(): %s" % str(nm_handler.check_correct_network()))
    log.debug("client_state.equal(CS_AUTH_SYNC): %s" % str(client_state.equal(CS_AUTH_SYNC)))

    if ((not nm_handler.check_correct_network()) or
        (not client_state.equal(CS_AUTH_SYNC))):
        # We are not connected. Exit and don't get called again.
        return False

    ceibal_conf = CeibalSCAConf.get_instance()

    current_seed = CurrentSeed()
    client_state.set_state("Timeout", CS_REAUTHNG_SYNC)
    while (client_state.equal(CS_REAUTHNG_SYNC)):
        try:
            current_seed = current_seed.load()
            
            if ( proto.check_public_url() ):
                # Cannot reach public_url, authenticate once more.
                proto.reauthenticate()
                current_seed = current_seed.load()
                client_state.set_state("Reauthenticated", CS_AUTH_SYNC)
                current_seed.setErrors(0)
                current_seed.save()
                reauth_timer.set_timer(current_seed.getSessionTimeout()*30, timer_handler)
                return False
            else:
                client_state.set_state("Not auth", CS_ACT_SYNC)
                connection_handler()

        except Exception, e:
            log.error(str(e))
            current_seed.incrErrors()
            if (current_seed.getErrors() >= current_seed.getMaxReauthFail()):
                client_state.set_state("Too many reauth errors", CS_ACT_DESYNC)
                current_seed.setErrors(0)
                current_seed.save()

                # Handle new state
                connection_handler()
                return False
            else:
                current_seed.save()
                time.sleep(ceibal_conf.get_reauth_sleep()) # Wait before retry

def reload_handler(sig_number, frame):
    """ Handler called when a SIGHUP signal is caught. This is used to
        reload configuration file.
    """
    ceibal_conf = CeibalSCAConf.get_instance()
    ceibal_conf.reread_conf()


# MAIN PROGRAM
# ============
if __name__ == '__main__':
    log = CeibalSCALog.get_instance()
    client_state = CeibalSCAState()
    reauth_timer = CeibalSCATimer.get_instance()

    try:

        signal.signal(signal.SIGHUP, reload_handler)
        proto = CeibalSCAProtocol()

        nm_handler = get_nm_handler(connection_handler, disconnection_handler)

        loop = gobject.MainLoop()
        loop.run()
    except BaseException, e:
        disconnection_handler()
        if (str(e) != ""):
            log.error(str(e))
        else:
            log.error("Uncaught exception. Exiting.")
